/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <string.h>
#include <stdlib.h>
#include "my_stdio.h"
#include "parse_tools.h"
#include "locks.h"

#ifndef LINE_LEN_CONF
#  define LINE_LEN_CONF 160
#endif


int read_conf_file(struct configuration **conf, char *filename)
{
	struct configuration *fc;
	FILE *fp;
	int count = 0;

	fp = fopen(filename, "r");
	if( fp == NULL ){
                SYSLOG(LOG_ERR,
                       "read_conf_file: Error opening file: %s",
		       filename);
                return -1;
        }

	if (*conf == NULL) {
		fc = *conf = (struct configuration *) 
			malloc(sizeof(struct configuration));
		fc->next = NULL;
	} else {
		for(fc = *conf; fc->next != NULL; fc = fc->next);
	}

	fc->type = FCT_EOF;
	memset(fc->line, 0, LINE_LEN_CONF);
	while( fgets(fc->line, LINE_LEN_CONF, fp)){
		count++;
		fc->line_num = count;
		fc->len = strlen(fc->line);
		if(fc->line[fc->len -1] == '\n')
			fc->line[fc->len -1] = 0;
		fc->sline = stripwhite(fc->line);
		if (parse_line(fc->sline, &fc->tag, 
			       &fc->data, &fc->comment) == 0){
			fc->type = FCT_TAG;
		} else if(fc->comment != NULL) {
			fc->type = FCT_COMMENT;
		} else if (*fc->sline == 0) {
			fc->type = FCT_NULL;
		} else {
			fc->type = FCT_UNKNOW;
			SYSLOG(LOG_ERR,"%s: ignoring line %d \"%s\"\n",
			      filename, fc->line_num, fc->line);
		}
		fc->next = (struct configuration *) 
			malloc(sizeof(struct configuration));
		fc = fc->next;
		fc->type = FCT_EOF;
		fc->next = NULL;
		memset(fc->line, 0, LINE_LEN_CONF);
	}

	fclose(fp);

	return 0;
}


int free_conf(struct configuration *conf)
{
	struct configuration *cf = conf;
	struct configuration *cfp;

	while(cf != NULL){
		cfp = cf;
		cf = cf->next;
		free(cfp);
	}

	return 0;
}


int write_conf_file(struct configuration *conf, char *filename)
{
	struct configuration *cf = conf;
	FILE *fp = stdout;
	char fmt[40];
	int c, i, t;
	int lock;
#if 1
	if ( (lock = if_lock_file(filename)) ) {
		PVERB(PERR,"File %s locked by pid: %d.\n", filename, lock);
		return -1;
	}

	fp = fopen(filename, "w");
	if( fp == NULL ){
                SYSLOG(LOG_ERR,
                       "write_conf_file: Error opening file: %s",
		       filename);
                return -1;
        }
#endif
	while (cf != NULL) {
		if (cf->type == FCT_TAG) {
			if ( cf->comment != NULL ) {
				c = sprintf(fmt,"%%s = %%s");
				i = 40-strlen(cf->tag)-strlen(cf->data)-3;
				t = i >> 3;
				if ( i & 0x07 ) t++;
				for(i=0; i < t; i++)
					c += sprintf(fmt+c,"\t");
				sprintf(fmt+c,"# %%s\n");
				fprintf(fp, fmt,
					cf->tag, cf->data, cf->comment);
			} else {
				fprintf(fp,"%s = %s\n", cf->tag, cf->data);
			}
		} else if (cf->type == FCT_COMMENT) {
			fprintf(fp,"#%s\n", cf->comment);
		} else if (cf->type == FCT_NULL) {
			fprintf(fp,"\n");
		} else if (cf->type != FCT_DELETED && cf->type != FCT_EOF) {
			fprintf(fp,"# [ignored: %s]\n", cf->sline);
		}
		cf = cf->next;
	}
	fclose(fp);
	unlock_file(filename);

	return 0;
}


struct configuration *find_conf_tag(struct configuration *conf, char *tag)
{
	struct configuration *cfp = conf;

	while(cfp != NULL) {
		if (cfp->type == FCT_TAG) {
			if (strcmp(cfp->tag, tag) == 0 ) {
				return cfp;
			}
		}
		cfp = cfp->next;
	}

	return (struct configuration *) NULL;
}


static int write_conf_line(char *line, char *tag, char *data, char *comment)
{
	int len;
	
	if ( comment != NULL ) {
		len = sprintf(line, "%s = %s # %s", tag, data, comment);
	} else {
		len = sprintf(line,"%s = %s", tag, data);
	}

	return len;
}


int mod_conf_data(struct configuration *cf, char *data)
{
	int ret = -1;
	char line[LINE_LEN_CONF];
	
	if (cf != NULL && cf->type == FCT_TAG) {
		//memset(line, 0, LINE_LEN_CONF);
		cf->len = write_conf_line(line, cf->tag, data, cf->comment);
		if (cf->len > LINE_LEN_CONF) cf->len = LINE_LEN_CONF;
		memset(cf->line, 0, LINE_LEN_CONF);
		strncpy(cf->line, line, cf->len);
		cf->sline = cf->line;
		if (parse_line(cf->line, &cf->tag, 
			       &cf->data, &cf->comment) == 0){
			ret = 0;
		}
	}

	return ret;
}


int change_conf_tag(struct configuration *conf, char *tag, char *data)
{
	int ret = -1;
	struct configuration *t = find_conf_tag(conf, tag);
	
	if (t != NULL) {
		ret = mod_conf_data(t, data);
	}

	return ret;
}


int add_conf_tag(struct configuration *conf, 
			  char *tag, char *data, char *comment)
{
	struct configuration *p = conf;

	while(p->next != NULL) p = p->next;
	if ( p->type != FCT_EOF )
		return -1;
	p->next = (struct configuration *)
		malloc(sizeof(struct configuration));
	p->type = FCT_TAG;
	memset(p->line, 0, LINE_LEN_CONF);
	p->len = write_conf_line(p->line, tag, data, comment);
	p->sline = p->line;
	parse_line(p->line, &p->tag, &p->data, &p->comment);

	p = p->next;
	p->type = FCT_EOF;
	p->next = NULL;


	return 0;
}


/* Only mark which FCT_DELETED, don't delete this */
int del_conf_tag(struct configuration *conf, char *tag)
{
	int ret = -1;
	struct configuration *t = find_conf_tag(conf, tag);
	
	if (t != NULL) {
		t->type = FCT_DELETED;
		ret = 0;
	}

	return ret;
}


int purge_conf_tag(struct configuration *conf)
{
	struct configuration *cf = conf;
	struct configuration *cfp = NULL;
	struct configuration *cfd;

	while (conf->type == FCT_DELETED) { /* los primeros */
		cfd = conf;
		conf = conf->next;
		free(cfd);
	}
	if(conf != NULL) {
		cfp = conf;
		cf = conf->next;
		while(cf != NULL){ /* el el medio */
			if (cf->type == FCT_DELETED) {
				cfd = cf;
				cfp->next = cf->next;
				cf = cf->next;
				free(cfd);
			} else {
				cfp = cf;
				cf = cf->next;
			}
		}
	}

	return 0;
}
